/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.google.gwt.jquery.mobile.ui;

import java.util.HashMap;
import java.util.Iterator;

import com.google.gwt.event.logical.shared.HasValueChangeHandlers;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.jquery.mobile.ui.face.HasIcon;
import com.google.gwt.jquery.mobile.ui.face.IsRefreshable;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.HasName;
import com.google.gwt.user.client.ui.Widget;

public class RadioGroup extends ControlGroup implements
		ValueChangeHandler<Boolean> {

	private boolean showIcons = true;
	private String iconPos = HasIcon.ICON_POS_LEFT;
	private HashMap<Widget, HandlerRegistration> handlers = new HashMap<Widget, HandlerRegistration>();

	public RadioGroup() {
		super();
		getElement().setId(DOM.createUniqueId());
	}

	@Override
	public void add(Widget w) {
		if (w instanceof HasValueChangeHandlers)
			handlers.put(w, ((HasValueChangeHandlers<Boolean>) w)
					.addValueChangeHandler(this));
		reloadChild(w);
		if (w instanceof HasName) {
			((HasName) w).setName(getElement().getId() + "-rdo");
		}
		super.add(w);
	}

	@Override
	public boolean remove(Widget w) {
		HandlerRegistration reg = handlers.remove(w);
		if (null != reg)
			reg.removeHandler();
		return super.remove(w);
	}

	protected void reloadChildren() {
		for (Iterator<Widget> iter = getChildren().iterator(); iter.hasNext();) {
			reloadChild(iter.next());
		}
	}

	protected void reloadChild(Widget w) {
		if (w instanceof HasIcon) {
			((HasIcon) w).setShowIcon(showIcons, iconPos);
		}
	}

	public void setIconPos(String iconPos) {
		this.iconPos = iconPos;
		reloadChildren();
	}

	public String getIconPos() {
		return this.iconPos;
	}

	public boolean isShowIcons() {
		return this.showIcons;
	}

	public void setShowIcons(boolean showIcon) {
		this.showIcons = showIcon;
		reloadChildren();
	}

	public void onValueChange(ValueChangeEvent<Boolean> event) {
		for (Iterator<Widget> iter = getChildren().iterator(); iter.hasNext();) {
			Widget w = iter.next();
			if (!w.equals(event.getSource()) && w instanceof IsRefreshable) {
				((IsRefreshable) w).refreshState();
			}
		}
	}
}