/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.google.gwt.jquery.mobile.ui;

import com.google.gwt.jquery.mobile.ui.face.HasCorners;
import com.google.gwt.jquery.mobile.ui.face.HasDataTheme;
import com.google.gwt.jquery.mobile.ui.face.HasMouseMovement;
import com.google.gwt.jquery.mobile.ui.util.CSSUtils;
import com.google.gwt.jquery.mobile.ui.util.JQueryMobile;
import com.google.gwt.user.client.ui.HTMLPanel;

public class StaticListItem extends HTMLPanel implements HasDataTheme, HasCorners, HasMouseMovement {

	private boolean initialized;
	private String dataTheme;
	private String roundingType;
	private String staticClass;
	private String defaultClass = "ui-li ui-li-static";
	
	public StaticListItem(String html) {
		super("li", html);
	}

	public void setRoundingType(String roundingType) {
		this.roundingType = roundingType;
		onStaticStyleNameChange(false);
	}

	public void setRounded(boolean rounded) {
		if (rounded) {
			setRoundingType(ROUND_TYPE_ALL);
		}
	}

	public String getRoundingType() {
		return roundingType;
	}

	public String getRoundingTypePrefix() {
		return "ui-corner";
	}

	public void setDataTheme(String dataTheme) {
		this.dataTheme = dataTheme;
		if (isAttached())
			CSSUtils.setStyleName(staticClass, this);
	}

	public String getDataTheme() {
		return dataTheme;
	}

	@Override
	protected void onAttach() {
		if (!initialized) {
			initialized = true;
			JQueryMobile.initWidget(this);
		}
		onStaticStyleNameChange(true);
		super.onAttach();
	}

	protected void onStaticStyleNameChange(boolean force) {
		if (isAttached() || force) {
			this.staticClass = CSSUtils.generateStaticStyleName(this,
					defaultClass);
			CSSUtils.setStyleName(staticClass, this);
		}
	}

	public String getDefaultThemedStyleName() {
		return "ui-btn-up";
	}

	public boolean isMouseDown() {
		return false;
	}

	public boolean isMouseOver() {
		return false;
	}

	public boolean isInteractive() {
		return false;
	}

	public String getThemedStyleName() {
		return "ui-btn";
	}
}