/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.google.gwt.jquery.mobile.ui;

import com.google.gwt.dom.client.Node;
import com.google.gwt.jquery.mobile.ui.base.ComplexPanel;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.Widget;

public class UlPanel extends ComplexPanel {
	  /**
	   * Creates an empty flow panel.
	   */
	  public UlPanel() {
	    setElement(DOM.createElement("ul"));
	  }

	  /**
	   * Adds a new child widget to the panel.
	   * 
	   * @param w the widget to be added
	   */
	  @Override
	  public void add(Widget w) {
	    add(w, getElement());
	  }

	  @Override
	  public void clear() {
	    try {
	      doLogicalClear();
	    } finally {
	      // Remove all existing child nodes.
	      Node child = getElement().getFirstChild();
	      while (child != null) {
	        getElement().removeChild(child);
	        child = getElement().getFirstChild();
	      }
	    }
	  }

	  /**
	   * Inserts a widget before the specified index.
	   * 
	   * @param w the widget to be inserted
	   * @param beforeIndex the index before which it will be inserted
	   * @throws IndexOutOfBoundsException if <code>beforeIndex</code> is out of
	   *           range
	   */
	  public void insert(Widget w, int beforeIndex) {
	    insert(w, getElement(), beforeIndex, true);
	  }
	}