/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.google.gwt.jquery.mobile.ui.base;

import com.google.gwt.jquery.mobile.ui.face.HasDataTheme;
import com.google.gwt.jquery.mobile.ui.util.CSSUtils;
import com.google.gwt.jquery.mobile.ui.util.JQueryMobile;
import com.google.gwt.user.client.ui.FlowPanel;

public abstract class AbstractFlowPanel extends FlowPanel implements HasDataTheme {

	private boolean initialized = false;
	private String dataTheme = null;
	private String defaultClass = null;

	public AbstractFlowPanel(String cssClass) {
		this.defaultClass = cssClass;
		onStaticStyleNameChange(false);
	}

	protected String getDefaultStyleName() {
		return defaultClass;
	}

	@Override
	public void addStyleName(String style) {
		super.addStyleName(style);
		defaultClass = CSSUtils.addClassName(defaultClass, style);
	}

	@Override
	public void removeStyleName(String style) {
		super.removeStyleName(style);
		defaultClass = CSSUtils.removeClassName(defaultClass, style);
	}

	protected void onStaticStyleNameChange(boolean force) {
		if (isAttached() || force) {
			CSSUtils.setStyleName(getDefaultStyleName(), this);
		}
	}

	protected String getThemedStyle() {
		return null;
	}

	public void setDataTheme(String dataTheme) {
		this.dataTheme = dataTheme;
		onStaticStyleNameChange(false);
	}

	public String getDataTheme() {
		return dataTheme;
	}

	@Override
	protected void onAttach() {
		if (!initialized) {
			JQueryMobile.initWidget(this);
			initialized = true;
		}
		onStaticStyleNameChange(true);
		super.onAttach();
	}
}