/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.google.gwt.jquery.mobile.ui.popup;

import com.google.gwt.core.client.GWT;
import com.google.gwt.jquery.mobile.ui.Button;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class Popup extends Composite implements HasPopupControls {

	private static PopupUiBinder uiBinder = GWT.create(PopupUiBinder.class);

	interface PopupUiBinder extends UiBinder<Widget, Popup> {
	}

	private PopupPanel backgroundPanel;

	@UiField
	PopupPanel popup;

	@UiField
	Label title;

	@UiField
	VerticalPanel contentPanel;

	@UiField
	Button okButton;

	@UiField
	Button cancelButton;

	public Popup() {
		backgroundPanel = new PopupPanel();
		addGlassStyle(backgroundPanel);
		initWidget(uiBinder.createAndBindUi(this));
	}

	public void close() {
		popup.hide();
		backgroundPanel.hide();
	}

	public void show() {
		backgroundPanel.show();
		popup.center();
	}

	private void addGlassStyle(PopupPanel glass) {
		// Plein écran
		DOM.setStyleAttribute(glass.getElement(), "width", "100%");
		DOM.setStyleAttribute(glass.getElement(), "height", "100%");
		// Fond noir
		DOM.setStyleAttribute(glass.getElement(), "backgroundColor", "#AAAAAA");
		// Transparence
		DOM.setStyleAttribute(glass.getElement(), "opacity", "0.70");
		DOM.setStyleAttribute(glass.getElement(), "filter",
				" alpha(opacity=70)");
		// Pas de bordure
		DOM.setStyleAttribute(glass.getElement(), "border", "none");
	}

	protected void onLoad() {
		this.show();
	}

	protected void onUnload() {
		this.close();
	}

	public void setContent(Widget w) {
		contentPanel.clear();
		contentPanel.add(w);
	}

	@Override
	public Button getCancelButton() {
		return cancelButton;
	}

	@Override
	public Button getOkButton() {
		return okButton;
	}

	@Override
	public String getPopupTitle() {
		return title.getText();
	}

	@Override
	public void setPopupTitle(String title) {
		this.title.setText(title);
	}
}
