/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.google.gwt.jquery.mobile.ui.popup;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.jquery.mobile.ui.util.MultiHashMap;
import com.google.gwt.jquery.mobile.ui.util.MultiMap;
import com.google.gwt.user.client.ui.HTML;

public class PopupManager {

	private static Popup currentPopup;

	private static List<Popup> waitingPopups = new ArrayList<Popup>();

	private static MultiMap<Popup, HandlerRegistration> handlerRegistrations = new MultiHashMap<Popup, HandlerRegistration>();

	public static void info(String title, String text) {
		final Popup popup = new Popup();
		popup.setPopupTitle(title);
		popup.getCancelButton().setVisible(false);
		HTML content = new HTML(text);
		popup.setContent(content);
		popup.getOkButton().setText("Ok");
		registerHandler(popup,
				popup.getOkButton().addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						popup.close();
						displayNextPopup();
					}
				}));
		addToDisplayQueue(popup);
	}

	public static void info(String title, String text, final BasePopupCallback callback) {
		final Popup popup = new Popup();
		popup.setPopupTitle(title);
		popup.getCancelButton().setVisible(false);
		HTML content = new HTML(text);
		popup.setContent(content);
		popup.getOkButton().setText("Ok");
		registerHandler(popup,
				popup.getOkButton().addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						callback.onOk();
						popup.close();
						displayNextPopup();
					}
				}));
		addToDisplayQueue(popup);
	}

	public static void confirm(String title, String text, final ConfirmPopupCallback callback) {
		final Popup popup = new Popup();
		popup.setPopupTitle(title);
		HTML content = new HTML(text);
		popup.setContent(content);
		popup.getOkButton().setText("Ok");
		registerHandler(popup,
				popup.getOkButton().addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						callback.onOk();
						popup.close();
						displayNextPopup();
					}
				}));
		registerHandler(popup,
				popup.getCancelButton().addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						callback.onCancel();
						popup.close();
						displayNextPopup();
					}
				}));
		addToDisplayQueue(popup);
	}

	public static void error(String title, String text) {
		final Popup popup = new Popup();
		popup.setPopupTitle(title);
		popup.getCancelButton().setVisible(false);
		HTML content = new HTML(text);
		popup.setContent(content);
		popup.getOkButton().setText("Ok");
		registerHandler(popup,
				popup.getOkButton().addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						popup.close();
						displayNextPopup();
					}
				}));
		addToDisplayQueue(popup);
	}

	public static void error(String title, String text, final BasePopupCallback callback) {
		final Popup popup = new Popup();
		popup.setPopupTitle(title);
		popup.getCancelButton().setVisible(false);
		HTML content = new HTML(text);
		popup.setContent(content);
		popup.getOkButton().setText("Ok");
		registerHandler(popup,
				popup.getOkButton().addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						callback.onOk();
						popup.close();
						displayNextPopup();
					}
				}));
		addToDisplayQueue(popup);
	}

	private static void addToDisplayQueue(Popup modalPopup) {
		if (currentPopup == null) {
			currentPopup = modalPopup;
			currentPopup.show();
		} else {
			waitingPopups.add(modalPopup);
		}
	}

	private static void displayNextPopup() {
		for (HandlerRegistration handler : handlerRegistrations
				.get(currentPopup)) {
			handler.removeHandler();
		}
		handlerRegistrations.remove(currentPopup);
		currentPopup = null;
		if (!waitingPopups.isEmpty()) {
			currentPopup = waitingPopups.remove(0);
			currentPopup.show();
		}
	}

	private static void registerHandler(Popup popup, HandlerRegistration handler) {
		handlerRegistrations.add(popup, handler);
	}

}
