/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.google.gwt.jquery.mobile.ui.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class MultiHashMap<K, V> extends HashMap<K, List<V>> implements
		MultiMap<K, V> {

	private static final long serialVersionUID = 3517689742297188682L;

	public boolean add(K key, V value) {
		if (!containsKey(key)) {
			put(key, new ArrayList<V>());
		}
		return get(key).add(value);
	}

	public boolean addAll(K key, List<V> values) {
		if (!containsKey(key)) {
			put(key, new ArrayList<V>());
		}
		return get(key).addAll(values);
	}

	public List<V> allValues() {
		List<V> result = new ArrayList<V>();
		for (List<V> list : values()) {
			result.addAll(list);
		}
		return result;
	}

	public boolean remove(K key, V item) {
		if (!containsKey(key)) {
			return false;
		}
		List<V> values = get(key);
		boolean result;
		try {
			result = values.remove(item);
		} catch (UnsupportedOperationException uoe) {
			result = false;
		}
		if (values.isEmpty()) {
			remove(key);
		}
		return result;
	}
}
